with Text_Io;	                                                                                        --  1
with Ada.Exceptions;	                                                                                  --  2
with Unchecked_Deallocation;	                                                                            --  3
package body Queue_Manager_1 is	                                                                      --  4
   -- This instantiation enables destruction of unreferenced allocated storage               	          --  5
   procedure Free_Node is new Unchecked_Deallocation	                                                    --  6
      (Object => List_Node,	                                                                            --  7
      Name   => Link);	                                                                                  --  8

   -- This instantiation enables destruction of unreferenced Data items	                                  --  9
   procedure Free_Item is new Unchecked_Deallocation	                                                    -- 10
      (Object => List_Item'Class,	                                                                      -- 11
      Name   => Item_Reference);	                                                                      -- 12

   -- We override Ada.Finalizaion for a single node	                                                    -- 13
   procedure Finalize(One_Node : in out List_Node) is	                                                    -- 14
   begin			                                                                                  -- 15
      Free_Item (One_Node.Data);	                                                                      -- 16
      Free_Node (One_Node.Next);	                                                                      -- 17
   end Finalize;	                                                                                        -- 18

   -- When the list goes out of scope, this is called to clean up the storage	                            -- 19
   procedure Finalize(The_List : in out List) is	                                                    -- 20
   begin			                                                                                  -- 21
      -- Use the Iterator to traverse the list and call Free_Item;  add this code yourself	          -- 22
      Free_Node (The_List.Current);	                                                                      -- 23
      Free_Node (The_List.Tail);	                                                                      -- 24
      Free_Node (The_List.Head);	                                                                      -- 25
   end Finalize;	                                                                                        -- 26

   -- The name says what it does.  Note the allocation of a temp. Finalization will 	                -- 27
   -- occur to ensure there is no left over storage.  	                                              -- 28
   procedure Insert_At_Head (L : in out List;	                                                          -- 29
         I : in     List_Item'Class) is	                                                                -- 30
      Temp_Item : Item_Reference := new List_Item'(I);	                                              -- 31
      Temp : Link := new List_Node'(Controlled with	                                                    -- 32
         Data => Temp_Item, 	                                                                            -- 33
         Next => null,	                                                                                  -- 34
         Prev => null);	                                                                                  -- 35
   begin			                                                                                  -- 36
      if Is_Empty(L'access) 	                                                                            -- 37
            then		                                                                                  -- 38
         L.Head := Temp;	                                                                            -- 39
         L.Tail := Temp;	                                                                            -- 40
      else		                                                                                        -- 41
         L.Head.Prev := Temp;	                                                                            -- 42
         Temp.Next := L.Head;	                                                                            -- 43
         L.Head := Temp;	                                                                            -- 44
      end if;	                                                                                        -- 45
      L.Count := L.Count + 1;	                                                                            -- 46
   end Insert_At_Head;	                                                                                  -- 47

   -- This is implemented in terms of the non-access version.  Simply makes it convenient	                -- 48
   -- to call this with access to object values, general or storage-pool access values.	                -- 49
   procedure Insert_At_Head (L : access List;	                                                          -- 50
         I : access List_Item'Class) is	                                                                -- 51
   begin			                                                                                  -- 52
      Insert_At_Head(L => L.all, 	                                                                      -- 53
         I => I.all);	                                                                                  -- 54
   end Insert_At_Head;	                                                                                  -- 55

   -- We implement this as a function instead of a procedure with in out modes	                      -- 56
   -- because this can be used in an expression to constrain a classwide variable	                      -- 57
   -- For example,  X : List_Item'Class := Remove(L);	                                                    -- 58
   function Remove_From_Tail (L : access List)	                                                          -- 59
         return List_Item'Class is	                                                                      -- 60
      Result : Item_Reference := L.Tail.Data;	                                                          -- 61
   begin			                                                                                  -- 62
      L.Tail := L.Tail.Prev;	                                                                            -- 63
      L.Count := L.Count - 1;	                                                                            -- 64
      Free_Item(L.Tail.Next.Data);	                                                                      -- 65
      Free_Node(L.Tail.Next);	                                                                            -- 66
      return Result.all;	                                                                            -- 67
   end Remove_From_Tail;	                                                                            -- 68

   -- You might want a more robust "=".  For example, it might be better to traverse	                -- 69
   -- each list, node by node, to ensure that each element is the same.  	                            -- 70
   function "=" (L, R : List) return Boolean is	                                                          -- 71
   begin			                                                                                  -- 72
      return L.Count = R.Count;	                                                                      -- 73
   end "=";		                                                                                        -- 74

   -- The name says it.  Simply returns how many nodes in this list.  	                                  -- 75
   function  Node_Count (L : access List) return Natural is	                                              -- 76
   begin			                                                                                  -- 77
      return L.Count;	                                                                                  -- 78
   end Node_Count;	                                                                                  -- 79

   -- This will not be correct unless you keep careful count of the inserted and deleted nodes.	          -- 80
   function  Is_Empty(L : access List) return Boolean is	                                              -- 81
   begin			                                                                                  -- 82
      return L.Count = 0;	                                                                            -- 83
   end Is_Empty;	                                                                                        -- 84

   -- We made List a limited private to prevent automatic assignment.  Instead, we design	                -- 85
   -- this "deep copy" procedure to ensure there will be two separate copies of the data	                -- 86
   procedure Copy (Source : in List;	                                                                -- 87
         Target : in out List) is	                                                                      -- 88
      type Item_Ref is access all List_Item'Class; 	                                                    -- 89
      Temp : Link := Source.Tail;   	                                                                -- 90
      Local_Data : Item_Reference;  	                                                                -- 91
   begin			                                                                                  -- 92
      Clear(Target);                                                                                      -- 92.1
      -- Be sure the target is initialized before copying.	                                              -- 93
      loop	                                                                                              -- 94
         exit when Temp = null;	                                                                      -- 95
         Local_Data := new List_Item'(Temp.Data.all);	                                                    -- 96
         declare	                                                                                        -- 97
            Local_List_Item 	                                                                            -- 98
               : List_Item'Class := Local_Data.all;	                                                    -- 99
         begin	                                                                                       -- 100
            Insert_At_Head(Target, Local_List_Item);	                                                   -- 101
         end;	                                                                                       -- 102
         Temp := Temp.Prev;	                                                                           -- 103
      end loop;	                                                                                       -- 104
   end Copy;	                                                                                       -- 105

   -- This is pretty simple.  It is also an important part of the overall design.	                     -- 106
   procedure Clear (L : in out List) is	                                                               -- 107
   begin			                                                                                 -- 108
      L.Head    := null;	                                                                           -- 109
      L.Tail    := null;	                                                                           -- 110
      L.Current := null;	                                                                           -- 111
      L.Count   := 0;	                                                                                 -- 112
   end Clear;	                                                                                       -- 113

   procedure Initialize_Iterator(This     : in out Iterator;	                                       -- 114
         The_List : access List) is	                                                                     -- 115
   begin			                                                                                 -- 116
      This := Iterator(The_List.Head);	                                                               -- 117
   end Initialize_Iterator;	                                                                           -- 118

   function Next(This : access Iterator) return Iterator is	                                             -- 119
   begin			                                                                                 -- 120
      return Next(This.all);	                                                                           -- 121
   end Next;	                                                                                       -- 122

   function Next (This : Iterator) return Iterator is	                                                   -- 123
   begin			                                                                                 -- 124
      return Iterator(This.Next);	                                                                     -- 125
   end Next;	                                                                                       -- 126

   function  Get (This : in Iterator) 	                                                               -- 127
         return List_Item'Class is	                                                                     -- 128
   begin			                                                                                 -- 129
      return This.Data.all;	                                                                           -- 130
   end Get;		                                                                                       -- 131

   function  Get (This : in Iterator) return Item_Reference is	                                       -- 132
   begin			                                                                                 -- 133
      return This.Data;	                                                                                 -- 134
   end Get;		                                                                                       -- 135

   function  Is_Done(This : in Iterator) return Boolean is	                                             -- 136
   begin			                                                                                 -- 137
      return This = null;	                                                                           -- 138
   end Is_Done;	                                                                                       -- 139

   function  Is_Done(This : access Iterator) 	                                                         -- 140
         return Boolean is	                                                                           -- 141
   begin			                                                                                 -- 142
      return Is_Done(This.all);	                                                                     -- 143
   end Is_Done;	                                                                                       -- 144
end Queue_Manager_1;	                                                                                 -- 145